
/*****************************************************************************
 *                                                                           *
 * This FILE mx12p_ui.c                                                      *
 * is part of the MX12P control panel program.  It is copyright (C) 1999     *
 * and 2000 by Gregory L. Kreider.  All rights reserved.  See below for      *
 * distribution terms.                                                       *
 *                                                                           *
 * The code in this file links a Tk user interface program (the actual       *
 * control panel) to the MX12P libraries.  More comments can be found in     *
 * mx12p_ui.doc.                                                             *
 *                                                                           *
 * This software package is provided "as is" without any further             *
 * warranties express or implied beyond those provided for by local law.     *
 * Neither the author nor any distributors are liable for any damages,       *
 * special, indirect, or consequential, incurred by the use or inability     *
 * to use this software, or for any claim by any other party.                *
 *                                                                           *
 * No further restrictions are placed on the use or distribution of this     *
 * code.                                                                     *
 *                                                                           *
 *****************************************************************************/



#include "mx12p_ui.h"
#include "mx12p_lib.h"

const int ErrMX_RC        = -10;  /* can't open ~/.mx12prc to save commport */
const int ErrMX_RCParse   = -11;  /* error parsing ~/.mx12prc */
const int ErrMX_NoConnect = -12;  /* no connection to MX12P */
const int ErrMX_SaveFile  = -13;  /* error handling save file */
const int ErrMX_FileName  = -14;  /* file name too long */

mx12p_globals mx12p_vars;
char* mx12p_fname;

TCamParameters mx12p_cam;


int setup_mx12p(void)
{
  char fname[256];
  FILE* fin;
  int status;

  /* initialize defaults, even though overridden when open PSU */
  mx12p_vars.MX_Used = 0;
  mx12p_vars.MX_CommPort = -1;
  mx12p_vars.MX_OpenPort = 0;
  mx12p_vars.MX_BlackEnable = 1;
  mx12p_vars.MX_BlackFine = 0;
  mx12p_vars.MX_BlackValue = 128;
  mx12p_vars.MX_Gain = 0;
  mx12p_vars.MX_Tint = 0;
  mx12p_vars.MX_Gamma = 1;
  mx12p_vars.MX_Mirror = 0;
  mx12p_vars.MX_Standard = 1;
  mx12p_vars.MX_Interlaced = 0;
  mx12p_vars.MX_Mode = 3;
  mx12p_vars.MX_Sync = 1;
  mx12p_vars.MX_Interface = 1;
  mx12p_vars.MX_Option1 = 1;
  mx12p_vars.MX_Option2 = 1;

  sprintf(fname,"%s/.mx12prc",getenv("HOME"));
  if ((fin = fopen(fname,"r")) == NULL) {
	/* do nothing */
  } else {
	  status = parseRCFile(fin);
	  fclose(fin);
  }

  PrepContextTable();
  return(0);
}


int closedown_mx12p(void)
{
  char fname[256];
  FILE *fout;
  int status;

  status = 0;
  if (mx12p_vars.MX_Used) {
	/* ignore failure here, want to write out port number if possible */
	CamClose(mx12p_vars.MX_OpenPort);
	sprintf(fname,"%s/.mx12prc",getenv("HOME"));
	if ((fout = fopen(fname,"w")) == NULL)
	  return(ErrMX_RC);
	status = writeRCFile(fout);
	fclose(fout);
  }
  return(status);
}
   

mx12p_globals* link_mx12p_vars(void)
{
  return(&mx12p_vars);
}


int closeMX12P(void)
{
  int status;

  if (!mx12p_vars.MX_Used)
	return(ErrMX_NoConnect);

  if ((status = CamClose(mx12p_vars.MX_OpenPort)) < 0)
	return(status);

  mx12p_vars.MX_Used = 0;
  mx12p_vars.MX_OpenPort = -1;
  mx12p_vars.MX_CommPort = -1;

  return(0);
}


int openMX12P(void)
{
  int status;

  if ((mx12p_vars.MX_Used) && 
	  (mx12p_vars.MX_CommPort != mx12p_vars.MX_OpenPort)) {
	if ((status = closeMX12P()))
	  return(status);
  }

  if ((status = CamOpen(mx12p_vars.MX_CommPort)) < 0)
	return(status);
  mx12p_vars.MX_Used = 1;
  mx12p_vars.MX_OpenPort = mx12p_vars.MX_CommPort;

  copyParToVar();
  return(0);
}


int transferToMX12P(void)
{
  int status;

  if (!mx12p_vars.MX_Used)
	return(ErrMX_NoConnect);

  /* prepare parameters --- no history, so copy everything */
  copyVarToPar();

  /* now do it */
  if ((status = CamTransfer(mx12p_vars.MX_OpenPort,&mx12p_cam)) < 0) {
	/* cleanup ? */
	return(status);
  }

  /* now copy the changed values back */
  mx12p_vars.MX_QPixelClock = mx12p_cam.QPixelClock;
  mx12p_vars.MX_QSensorType = mx12p_cam.QSensorType;

  return(0);
}


int attentionMX12P(void)
{

  if (!mx12p_vars.MX_Used)
	return(ErrMX_NoConnect);

  return(CamAttention(mx12p_vars.MX_OpenPort));
}


int readFromPSU(void)
{
  int status;

  if (!mx12p_vars.MX_Used)
	return(ErrMX_NoConnect);
  if ((status = CamLoadDefault(mx12p_vars.MX_OpenPort,&mx12p_cam)) < 0)
	return(status);

  copyParToVar();
  return(0);
}


int writeToPSU(void)
{
  int status;

  if (!mx12p_vars.MX_Used)
	return(ErrMX_NoConnect);

  copyVarToPar();
  if ((status = CamSaveAsDefault(mx12p_vars.MX_OpenPort,&mx12p_cam)) < 0) {
	closeMX12P();
	return(status);
  }
	
  return(0);
}


int readFromFile(void)
{
  int c;
  char fname[256],line[256];
  FILE* fin;

  if (strlen(mx12p_fname) > 255)
	return(ErrMX_FileName);
  sprintf(fname,"%s",mx12p_fname);

  if ((fin = fopen(fname,"r")) == NULL)
	return(ErrMX_SaveFile);

  while (fgets(line,256,fin) != NULL) {
	if (!strncmp(line,"#",1)) {
	  /* comment */
	} else if (!strncmp(line,"MX_BlackEnable",17)) {
	  c = 17;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_BlackEnable);
	} else if (!strncmp(line,"MX_BlackFine",15)) {
	  c = 15;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_BlackFine);
	} else if (!strncmp(line,"MX_BlackValue",16)) {
	  c = 16;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_BlackValue);
	} else if (!strncmp(line,"MX_Gain",10)) {
	  c = 10;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Gain);
	} else if (!strncmp(line,"MX_Tint",10)) {
	  c = 11;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Tint);
	} else if (!strncmp(line,"MX_Gamma",11)) {
	  c = 11;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Gamma);
	} else if (!strncmp(line,"MX_Mirror",12)) {
	  c = 12;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Mirror);
	} else if (!strncmp(line,"MX_Standard",14)) {
	  c = 14;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Standard);
	} else if (!strncmp(line,"MX_Interlaced",16)) {
	  c = 16;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Interlaced);
	} else if (!strncmp(line,"MX_Mode",10)) {
	  c = 10;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Mode);
	} else if (!strncmp(line,"MX_Sync",10)) {
	  c = 10;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Sync);
	} else if (!strncmp(line,"MX_Interface",15)) {
	  c = 15;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Interface);
	} else if (!strncmp(line,"MX_Option1",13)) {
	  c = 13;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Option1);
	} else if (!strncmp(line,"MX_Option2",13)) {
	  c = 13;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_Option2);
	} else if (!strncmp(line,"MX_QPixelClock",17)) {
	  c = 17;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_QPixelClock);
	} else if (!strncmp(line,"MX_QSensorType",17)) {
	  c = 17;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c < 256)
		sscanf(line+c,"%d",&mx12p_vars.MX_QSensorType);
	} /* ignore other lines */
  }
  fclose(fin);

  return(transferToMX12P());
}


int writeToFile(void)
{
  char fname[256];
  time_t t;
  FILE* fout;

  if (strlen(mx12p_fname) > 255)
	return(ErrMX_FileName);
  sprintf(fname,"%s",mx12p_fname);

  if ((fout = fopen(fname,"w")) == NULL)
	return(ErrMX_SaveFile);
  t = 0;
  time(&t);
  fprintf(fout,"# MX12P Configuration File, saved %s",ctime(&t));
  fprintf(fout,"%-20s %02d\n","MX_BlackEnable",mx12p_vars.MX_BlackEnable);
  fprintf(fout,"%-20s %02d\n","MX_BlackFine",mx12p_vars.MX_BlackFine);
  fprintf(fout,"%-20s %03d\n","MX_BlackValue",mx12p_vars.MX_BlackValue);
  fprintf(fout,"%-20s %03d\n","MX_Gain",mx12p_vars.MX_Gain);
  fprintf(fout,"%-20s %03d\n","MX_Tint",mx12p_vars.MX_Tint);
  fprintf(fout,"%-20s %02d\n","MX_Gamma",mx12p_vars.MX_Gamma);
  fprintf(fout,"%-20s %02d\n","MX_Mirror",mx12p_vars.MX_Mirror);
  fprintf(fout,"%-20s %02d\n","MX_Standard",mx12p_vars.MX_Standard);
  fprintf(fout,"%-20s %02d\n","MX_Interlaced",mx12p_vars.MX_Interlaced);
  fprintf(fout,"%-20s %02d\n","MX_Mode",mx12p_vars.MX_Mode);
  fprintf(fout,"%-20s %02d\n","MX_Sync",mx12p_vars.MX_Sync);
  fprintf(fout,"%-20s %02d\n","MX_Interface",mx12p_vars.MX_Interface);
  fprintf(fout,"%-20s %02d\n","MX_Option1",mx12p_vars.MX_Option1);
  fprintf(fout,"%-20s %02d\n","MX_Option2",mx12p_vars.MX_Option2);
  fprintf(fout,"%-20s %06d\n","MX_QPixelClock",mx12p_vars.MX_QPixelClock);
  fprintf(fout,"%-20s %06d\n","MX_QSensorType",mx12p_vars.MX_QSensorType);
  fclose(fout);

  return(0);
}


int parseRCFile(FILE* fin)
{
  int c;
  char line[256];

  while (fgets(line,256,fin) != NULL) {
	if (!strncmp(line,"#",1)) {
	  /* comment */
	} else if (!strncmp(line,"LastCommPort",12)) {
	  c = 12;
	  while (isspace(line[c]) && (c < 256))
		c++;
	  if (c == 256)
		return(ErrMX_RCParse);
	  if (sscanf(line+c,"%d",&mx12p_vars.MX_CommPort) != 1)
		return(ErrMX_RCParse);
	} else {
	  return(ErrMX_RCParse);
	}
  }
  return(0);
}


int writeRCFile(FILE* fout) {
  fprintf(fout,"%-20s %02d\n","LastCommPort",mx12p_vars.MX_OpenPort);
  return(0);
}


void copyVarToPar(void)
{
  mx12p_cam.Gamma = mx12p_vars.MX_Gamma;
  mx12p_cam.Mirror = mx12p_vars.MX_Mirror;
  mx12p_cam.Sync = mx12p_vars.MX_Sync;
  mx12p_cam.Interlace = mx12p_vars.MX_Interlaced;
  mx12p_cam.Black = mx12p_vars.MX_BlackEnable;
  mx12p_cam.BlackFine = mx12p_vars.MX_BlackFine;
  mx12p_cam.IntegrationTime = mx12p_vars.MX_Tint;
  mx12p_cam.Gain = mx12p_vars.MX_Gain;
  mx12p_cam.BlackValue = mx12p_vars.MX_BlackValue;
  mx12p_cam.Mode = mx12p_vars.MX_Mode;
  mx12p_cam.Standard = mx12p_vars.MX_Standard;
  mx12p_cam.Interface = mx12p_vars.MX_Interface;
  mx12p_cam.Option_1 = mx12p_vars.MX_Option1;
  mx12p_cam.Option_2 = mx12p_vars.MX_Option2;
  mx12p_cam.Invalidate = 1;
}


void copyParToVar(void)
{
  mx12p_vars.MX_Gamma = mx12p_cam.Gamma;
  mx12p_vars.MX_Mirror = mx12p_cam.Mirror;
  mx12p_vars.MX_Sync = mx12p_cam.Sync;
  mx12p_vars.MX_Interlaced = mx12p_cam.Interlace;
  mx12p_vars.MX_BlackEnable = mx12p_cam.Black;
  mx12p_vars.MX_BlackFine = mx12p_cam.BlackFine;
  mx12p_vars.MX_Tint = mx12p_cam.IntegrationTime;
  mx12p_vars.MX_Gain = mx12p_cam.Gain;
  mx12p_vars.MX_BlackValue = mx12p_cam.BlackValue;
  mx12p_vars.MX_Mode = mx12p_cam.Mode;
  mx12p_vars.MX_Standard = mx12p_cam.Standard;
  mx12p_vars.MX_Interface = mx12p_cam.Interface;
  mx12p_vars.MX_Option1 = mx12p_cam.Option_1;
  mx12p_vars.MX_Option2 = mx12p_cam.Option_2;

  mx12p_vars.MX_QPixelClock = mx12p_cam.QPixelClock;
  mx12p_vars.MX_QSensorType = mx12p_cam.QSensorType;
}


